#include <sstream>
#include <iostream>
#include <fstream>
#include <vector>
#include "ParametricShape.h"

static bool write_stl(std::string filename, const std::vector<Eigen::Vector3d> &vertices, const std::vector<Eigen::Vector3i> &triangles)
{

	FILE *  f = fopen(filename.c_str(), "wb");

	char header[80];
	memset(header, ' ', 80);
	fwrite(header, 80, 1, f);

	int nfaces = triangles.size();
	fwrite(&nfaces, 4, 1, f);

	for (size_t i = 0; i < triangles.size(); i++) {
		float fbuf[12];
		fbuf[0] = 0; fbuf[1] = 1; fbuf[2] = 0;
		fbuf[3]  = vertices[triangles[i](0)](0);
		fbuf[4]  = vertices[triangles[i](0)](1);
		fbuf[5]  = vertices[triangles[i](0)](2);
		fbuf[6]  = vertices[triangles[i](1)](0);
		fbuf[7]  = vertices[triangles[i](1)](1);
		fbuf[8]  = vertices[triangles[i](1)](2);
		fbuf[9]  = vertices[triangles[i](2)](0);
		fbuf[10] = vertices[triangles[i](2)](1);
		fbuf[11] = vertices[triangles[i](2)](2);
		fwrite(fbuf, 48, 1, f);
		unsigned char att[2] = { 0, 0 };
		fwrite(att, 2, 1, f);
	}
	return true;
}




int main(int argc, char* argv[]){
	
	std::string inFilename;
	std::string outFilename;
	double finiteDiff;

	if (argc < 5) { // Check the value of argc. If not enough parameters have been passed, inform user and exit.
		std::cout << "Usage is .\example_main -i <input> (one of the parametric shape) -o <output> (stl file) \n"; // Inform the user of how to use the program
		system("pause");
		exit(0);
	}
	else {

		int numberOfInputs = 0;
		for (int i = 0; i < argc; i++) {
			if (std::string(argv[i]) == "-i") 
			{
				inFilename = std::string(argv[i + 1]);
				i++;
				numberOfInputs++;
			}
			else if (std::string(argv[i]) == "-o") 
			{
				outFilename = std::string(argv[i + 1]);
				i++;
				numberOfInputs++;
			}
		}
	}

	ParametricShape shape(inFilename);
	std::vector<Eigen::Vector3d> ver;
	std::vector<Eigen::Vector3i> tri;
	shape.evalShape(shape.getDefaultParams(), &ver, &tri);


	write_stl(outFilename, ver, tri);


	system("pause"); 

}




